#!/usr/bin/env php
<?php
/**
* Script to refresh the pending alarm times for the next alarm instance.
*
* @package   davical
* @subpackage   alarms
* @author    Andrew McMillan <andrew@morphoss.com>
* @copyright Morphoss Ltd
* @license   http://gnu.org/copyleft/gpl.html GNU GPL v3 or later
*/
$script_file = __FILE__;

chdir(str_replace('/scripts/refresh-alarms.php','/htdocs',$script_file));
$_SERVER['SERVER_NAME'] = 'localhost';

/**
* Call with something like e.g.:
*
* scripts/refresh_alarms.php -p P1800D -f P1D
*
*/

$args = (object) array();
#ifdef SYNO
//$args->debug = false;
$args->debug = true;
#endif
$args->set_last = false;

$args->future = 'P400D';
$args->near_past = 'P1D';
$debugging = null;
const UTC_FORMAT = 'Ymd\THis\Z';


function parse_arguments() {
  global $args;

  $opts = getopt( 'f:p:s:d:lh' );
  foreach( $opts AS $k => $v ) {
    switch( $k ) {
      case 'f':   $args->future = $v;  break;
      case 'p':   $args->near_past = $v;  break;
      case 's':   $_SERVER['SERVER_NAME'] = $v; break;
      case 'd':   $args->debug = true;  $debugging = explode(',',$v); break;
      case 'l':   $args->set_last = true; break;
      case 'h':   usage();  break;
      default:    $args->{$k} = $v;
    }
  }
}

function usage() {

  echo <<<USAGE
Usage:
   refresh-alarms.php [-s server.domain.tld] [other options]

  -s <server>      The servername to be used to identify the DAViCal configuration file.
  -p <duration>    Near past period to review for finding recently last instances: default 1 days ('P1D')
  -f <duration>    Future period to consider for finding future alarms: default ~5 years ('P2000D')

  -l               Try to set the 'last' alarm date in historical alarms

  -d xxx           Enable debugging where 'xxx' is a comma-separated list of debug subsystems

USAGE;
  exit(0);
}

parse_arguments();

if ( $args->debug && is_array($debugging )) {
  foreach( $debugging AS $v ) {
    $c->dbg[$v] = 1;
  }
}
$args->near_past = '-' .  $args->near_past;

require_once("./always.php");
require_once('AwlQuery.php');
require_once('RRule-v2.php');
require_once('vCalendar.php');


/**
* Essentially what we are doing is:
*
UPDATE calendar_alarm
  SET next_trigger = (SELECT rrule_event_instances_range(
                        dtstart + icalendar_interval_to_SQL(replace(trigger,'TRIGGER:','')),
                        rrule,
                        current_timestamp, current_timestamp + '2 days'::interval,
                        1)
                     LIMIT 1)
 FROM calendar_item
WHERE calendar_alarm.dav_id = calendar_item.dav_id
  AND next_trigger is null
  AND rrule IS NOT NULL

*/
$expand_range_start = new RepeatRuleDateTime(gmdate('Ymd\THis\Z'));
#ifdef SYNO
/*
$expand_range_end   = new RepeatRuleDateTime(gmdate('Ymd\THis\Z'));
$expand_range_end->modify( $args->future );
*/
#endif



$earliest   = clone($expand_range_start);
$earliest->modify( $args->near_past );

$now = new RepeatRuleDateTime(gmdate('Ymd\THis\Z'));
printf("now=" . print_r($now, true));
$earliest = clone($now);
//$expand_range_end = clone($now);

$sql = 'SELECT * FROM calendar_alarm JOIN calendar_item USING (dav_id) JOIN caldav_data USING (dav_id) JOIN syno_user_setting on calendar_item.user_no = syno_user_setting.user_no WHERE rrule IS NOT NULL AND next_trigger IS NULL';
if ( $args->debug ) printf( "%s\n", $sql );
$qry = new AwlQuery( $sql );
if ( $qry->Exec() && $qry->rows() ) {

  while( $alarm = $qry->Fetch() ) {
    if ( $args->debug ) printf( "refresh: Processing alarm for '%s' based on '%s','%s', '%s', '%s'\n",
                          $alarm->dav_name, $alarm->dtstart, $alarm->rrule, $alarm->trigger, $alarm->time_zone );
    $ic = new vComponent( $alarm->caldav_data );
	$range = getVCalendarRange($ic);
	printf("alarm->rrule=" . print_r($alarm->rrule, true) . "\n");
	//printf("range=" . print_r($range, true) . "\n");
    $expanded = expand_event_instances2( $ic, 5, $now);
    $expanded->MaskComponents( array( 'VEVENT'=>1, 'VTODO'=>1, 'VJOURNAL'=>1 ) );
    $instances = $expanded->GetComponents();

    $trigger = new vProperty( 'TRIGGER');
    $trigger->ParseFrom($alarm->trigger);
    $related = $trigger->GetParameterValue('RELATED');

    $first = new RepeatRuleDateTime($alarm->dtstart);
    $first->modify( $trigger->Value() );
    $next = null;
    $last = null;

    $trigger_type = $trigger->GetParameterValue('VALUE');
    if ( $trigger_type == 'DATE' || $trigger_type == 'DATE-TIME' || preg_match('{^\d{8}T\d{6}Z?$}', $trigger->Value()) ) {
      $first = new RepeatRuleDateTime($trigger);
      if ( $first->UTC(UTC_FORMAT, $alarm->time_zone) > $expand_range_start->UTC(UTC_FORMAT, $alarm->time_zone) && (empty($next) || $first->UTC(UTC_FORMAT, $alarm->time_zone) < $next->UTC(UTC_FORMAT, $alarm->time_zone) ) )
        $next = $first;
      else if ( empty($next) ) {
        if ( $args->set_last && (empty($last) || $first->UTC(UTC_FORMAT, $alarm->time_zone) > $last->UTC(UTC_FORMAT, $alarm->time_zone)) )
          $last = $first;
      }
    }
    if ( $args->set_last && !isset($last) && (!isset($next) || $next->UTC(UTC_FORMAT, $alarm->time_zone) < $expand_range_Start->UTC(UTC_FORMAT, $alarm->time_zone)) ) {
      if ( isset($range->until) && $range->until->UTC(UTC_FORMAT, $alarm->time_zone) < $earliest->UTC(UTC_FORMAT, $alarm->time_zone) )
        $last = $range->until;
    }

    // Find & Fill next_trigger of sub-events
    if ($alarm->recurrence_id != '') {
      $cs = $ic->GetComponents();
      foreach($cs AS $k => $comp) {
        if ( $comp->GetType() != 'VEVENT' && $comp->GetType() != 'VTODO' && $comp->GetType() != 'VJOURNAL' ) {
          continue;
        }
        $recurrence_id = $comp->GetProperty('RECURRENCE-ID');
        if ($recurrence_id == '' || false == strpos($recurrence_id, $alarm->recurrence_id)) {
          continue;
        }
        printf("recurrence-id= " . $recurrence_id . "\n");

        $when = new RepeatRuleDateTime( $comp->GetPValue('DTSTART') ); // a UTC value
        if ( $related == 'END' ) {
          $when->modify( $component->GetPValue('DURATION') );
        }
        $when->modify( $trigger->Value() );

        if ( $args->debug ) printf( "refresh: Found next alarm instance on '%s' of subevent\n", $when->UTC(UTC_FORMAT, $alarm->time_zone) );
        $sql = 'UPDATE calendar_alarm SET next_trigger = :next WHERE dav_id = :id AND component = :component AND recurrence_id = :recurrence_id';
        $update = new AwlQuery( $sql, array( ':next' => $when->UTC(UTC_FORMAT, $alarm->time_zone), ':id' => $alarm->dav_id, ':component' => $alarm->component, ':recurrence_id' => $alarm->recurrence_id ) );
        $update->Exec('refresh-alarms', __LINE__, __FILE__ );
      }
      continue;
    }

    // Find next_trigger of repeat events
    foreach( $instances AS $k => $component ) {
      if (!!($component->GetProperty('IS_OVERRIDDEN'))) {
        continue;
      }

      $when = new RepeatRuleDateTime( $component->GetPValue('DTSTART') ); // a UTC value
      if ( $args->debug ) printf( "refresh: Looking at event instance on '%s'\n", $when->UTC(UTC_FORMAT, $alarm->time_zone) );
      if ( $related == 'END' ) {
        $when->modify( $component->GetPValue('DURATION') );
      }
      $when->modify( $trigger->Value() );
      if ( $when->UTC(UTC_FORMAT, $alarm->time_zone) > $expand_range_start->UTC(UTC_FORMAT, $alarm->time_zone) && (!isset($next) || $when->UTC(UTC_FORMAT, $alarm->time_zone) < $next->UTC(UTC_FORMAT, $alarm->time_zone)) ) {
        $next = clone($when);
      }
      if ( $args->set_last && (!isset($last) || $when->UTC(UTC_FORMAT, $alarm->time_zone) > $last->UTC(UTC_FORMAT, $alarm->time_zone)) ) {
        $last = clone($when);
      }
    }

    // Fill next_trigger of repeat events
    if ( isset($next) && (!isset($range->until) || $next->UTC(UTC_FORMAT, $alarm->time_zone) < $range->until->UTC(UTC_FORMAT, $alarm->time_zone))) {
      if ( $args->debug ) printf( "refresh: Found next alarm instance on '%s'\n", $next->UTC(UTC_FORMAT, $alarm->time_zone) );
      $sql = 'UPDATE calendar_alarm SET next_trigger = :next WHERE dav_id = :id AND component = :component AND (recurrence_id = \'\') IS NOT FALSE';
      $update = new AwlQuery( $sql, array( ':next' => $next->UTC(UTC_FORMAT, $alarm->time_zone), ':id' => $alarm->dav_id, ':component' => $alarm->component ) );
      $update->Exec('refresh-alarms', __LINE__, __FILE__ );
    }
    else if ( $args->set_last && isset($last) && $last->UTC(UTC_FORMAT, $alarm->time_zone) < $earliest->UTC(UTC_FORMAT, $alarm->time_zone) ) {
      if ( $args->debug ) printf( "refresh: Found past final alarm instance on '%s'\n", $last->UTC(UTC_FORMAT, $alarm->time_zone) );
      $sql = 'UPDATE calendar_alarm SET next_trigger = :last WHERE dav_id = :id AND component = :component AND (recurrence_id = \'\') IS NOT FALSE';
      $update = new AwlQuery( $sql, array( ':last' => $last->UTC(UTC_FORMAT, $alarm->time_zone), ':id' => $alarm->dav_id, ':component' => $alarm->component ) );
      $update->Exec('refresh-alarms', __LINE__, __FILE__ );
    }
	else {
		//save yesterday to for it have been check
		$now->modify('-P1D');
        $sql = 'UPDATE calendar_alarm SET next_trigger = :last WHERE dav_id = :id AND component = :component AND (recurrence_id = \'\') IS NOT FALSE';
		$update = new AwlQuery( $sql, array( ':last' => $now->UTC(UTC_FORMAT, $alarm->time_zone), ':id' => $alarm->dav_id, ':component' => $alarm->component ) );
		$update->Exec('refresh-alarms', __LINE__, __FILE__ );
	}
  }
}

