'use strict';
var ICSGenerator = require('../../jslib/evt_adapter/index.js');
var logger = require('../../jslib/logger/command.js');

function pad(value, length) {
	return (value.toString().length < length) ? pad("0"+value, length):value;
}

function getUTCDateTime(jsDate) {
	return `${pad(jsDate.getUTCFullYear(), 4)}-${pad(jsDate.getUTCMonth()+1, 2)}-${pad(jsDate.getUTCDate(), 2)}T${pad(jsDate.getUTCHours(), 2)}:${pad(jsDate.getUTCMinutes(), 2)}:${pad(jsDate.getUTCSeconds(), 2)}`
}

function formDT(gTimeObj, defaultTimeZone, tzidList) {
	var icalTime,
		gTime = gTimeObj.date || gTimeObj.dateTime,
		isAllDay = gTimeObj.date ? true : false,
		tz = gTimeObj.timeZone || defaultTimeZone,
		gDt;

	if (isAllDay === false && !tz) {
		console.error("non-allday event should have timezone");
	}
	if (tz) {
		tzidList.add(tz);
	}
	if (isAllDay) {
		icalTime = gTime.replace(/-/g, '');
	} else {
		gDt = new Date(gTime);
		icalTime = ICAL.Time.fromDateTimeString(getUTCDateTime(gDt)+'Z');
		icalTime = icalTime.convertToZone(ICAL.TimezoneService.get(tz));
		icalTime = icalTime.toICALString();
	}
	return (isAllDay ? '' : `TZID=${tz}:`) + icalTime;
}

function escapeNewline(str) {
	return str.replace(/\r/g, '\\r').replace(/\n/g, '\\n');
}

function formCommonData(event, defaultTimeZone, tzidList) {
	var isAllDay = event.start.date ? true : false,
		gStart = event.start,
		gEnd = event.end;

	var data = {
		summary: event.summary || '',
		transp: 'transparent' === event.transparency ? 'TRANSPARENT' : 'OPAQUE',
		dtstart: formDT(gStart, defaultTimeZone, tzidList),
		dtend: formDT(gEnd, defaultTimeZone, tzidList),
		description: event.description || '',
		location: event.location || '',
		sequence: event.sequence || 0,
		uid: event.id, //ICSGenerator.generateGUID(),
		is_all_day: isAllDay
	};

	data.summary = escapeNewline(data.summary);
	data.description = escapeNewline(data.description);
	data.location = escapeNewline(data.location);

	if (event.recurrence && event.recurrence.length > 0) {
		data.is_repeat_evt = true;
		event.recurrence.forEach(function(recurrenceItem) {
			if (0 === recurrenceItem.indexOf('RRULE')) {
				data.repeat_rule = recurrenceItem.replace('RRULE:', '');
			} else if (0 === recurrenceItem.indexOf('EXDATE')) {
				data.exdate = recurrenceItem;
			}
		});
	}

	return data;
}

function formAlarmList(reminders) {
	var alarmList = [];

	if (!Array.isArray(reminders)) {
		return alarmList;
	}

	reminders.forEach(function(obj) {
		var alarm = {
			alarm_action: 'DISPLAY',
			time_format: 'DURATION',
			time_value: ICSGenerator.convertToHourDay(obj.minutes)
		};
		alarmList.push(alarm);
	});
	return alarmList;
}

function addAlarm(vevent, alarmList) {
	if (!Array.isArray(alarmList)) {
		return;
	}
	alarmList.forEach(function(alarmObj) {
		var valarm = ICAL.Component.fromString(ICSGenerator.formAlarm(alarmObj));
		vevent.addSubcomponent(valarm);
	});
}

function addAttendee(vevent, attendeeList, organizer) {
	if (!Array.isArray(attendeeList) || !organizer) {
		return;
	}
	var action = 'NEEDS-ACTION';
	vevent.addProperty(ICAL.Property.fromString(ICSGenerator.formOrganizeIcal(organizer.email)));
	attendeeList.forEach(function(attendee) {
		if (attendee.responseStatus) {
			action = attendee.responseStatus.toUpperCase();
		}
		vevent.addProperty(ICAL.Property.fromString(ICSGenerator.formAttendeeIcal(attendee.email, action, action === 'NEEDS-ACTION' ? '1.2' : '2.0', organizer.email)));
	});
}

function getExdateList(events, defaultTimeZone, tzidList) {
	/* get deleted event list and remove it from events*/
	var exdateList = [];
	events.forEach(function(event) {
		if (event.status === 'cancelled') {
			if (event.originalStartTime) {
				var gTime = event.originalStartTime;
				exdateList.push(formDT(gTime, defaultTimeZone, tzidList));
			}
		}
	});
	return exdateList;
}

function addRecurrenceId(vevent, event, defaultTimeZone, tzidList) {
	var gTimeObj = event.originalStartTime,
		isAllDay = gTimeObj.date ? true : false,
		tz = gTimeObj.timeZone || defaultTimeZone,
		icalTime = ICSGenerator.fromAllString(formDT(gTimeObj, defaultTimeZone, tzidList));

	vevent.addPropertyWithValue('recurrence-id', icalTime);
	var prop = vevent.getFirstProperty('recurrence-id');
	if (!isAllDay) {
		prop.setParameter('tzid', tz);
	}
}

function handleListEventResponse(response, repeatEventList, tzidList) {
	var events = response.items,
		data = {},
		alarmList = [],
		defaultTimeZone = response.timeZone,
		vevents = '',
		strVevent = '',
		vevent,
		defaultReminders = response.defaultReminders;

	if (events.length === 0) {
		logger.debug('No upcoming events found.');
	} else {
		events.forEach( function (event) {
			if (event.recurrence && event.recurrence.length > 0) {
				repeatEventList.push(event.iCalUID);
				return;
			} else if (event.recurringEventId) {
				//it is a subevent, skip it
				return;
			}

			data = formCommonData(event, defaultTimeZone, tzidList);

			strVevent = ICSGenerator.formVEvent(data);
			vevent = ICAL.Component.fromString(strVevent);

			//add ALARM
			alarmList = event.reminders.useDefault ? formAlarmList(defaultReminders) : formAlarmList(event.reminders.overrides);
			addAlarm(vevent, alarmList);
			if (event.attendees && event.organizer) {
				addAttendee(vevent, event.attendees, event.organizer);
			}

			vevents += vevent.toString() + '\r\n';
		}, this);
	}
	return vevents;
}

function handleRepeatEventResponse(response, tzidList) {
	var events = response.items,
		data = {},
		vevents = '',
		mainEventId = '',
		exdateList = [],
		alarmList = [],
		defaultTimeZone = response.timeZone,
		defaultReminders = response.defaultReminders;

	if (events.length === 0) {
		logger.debug('No upcoming events found.');
	} else {
		try {
			exdateList = getExdateList(events, defaultTimeZone, tzidList);
			events.forEach(function(event) {
				var strVevent,
					vevent;

				if (event.status === 'cancelled') {
					return;
				}

				data = formCommonData(event, defaultTimeZone, tzidList);

				strVevent = ICSGenerator.formVEvent(data);
				vevent = ICAL.Component.fromString(strVevent);

				//update subevent uid and add recurrence-id
				if (event.originalStartTime) {
					ICSGenerator.updateUID(vevent, mainEventId);
					addRecurrenceId(vevent, event, defaultTimeZone, tzidList);
				}
				//add ALARM
				alarmList = event.reminders.useDefault ? formAlarmList(defaultReminders) : formAlarmList(event.reminders.overrides);
				addAlarm(vevent, alarmList);
				//add EXDATE to main event
				if (data.is_repeat_evt) {
					mainEventId = data.uid;
					if (data.exdate) {
						vevent.addProperty(ICAL.Property.fromString(data.exdate));
					}
					exdateList.forEach(function(exdate) {
						ICSGenerator.updateExdate(vevent, ICSGenerator.fromAllString(exdate).convertToZone(ICAL.Timezone.utcTimezone));
					});
				}
				vevents += (vevent.toString() + "\r\n");
			});
		} catch(err) {
			logger.debug(err);
		}
		return vevents;
	}
}

module.exports = {
	formDT: formDT,
	formAlarmList: formAlarmList,
	formCommonData: formCommonData,
	addAlarm: addAlarm,
	handleRepeatEventResponse: handleRepeatEventResponse,
	handleListEventResponse: handleListEventResponse
};
