'use strict';
var ICAL = require('ical.js');
var OS = require('os');
var assert = require('assert');
var fs = require('fs');
var path = require('path');
var logger = require('../logger/command.js');

var timeParser = {
	convertToHourDay: function(time) {
		var m = 0,
			h = 0,
			d = 0;
		if (time > 0) {
			m = time % 60;
		}
		time -= m;
		time = time/60;
		if (time > 0) {
			h = time % 24;
		}
		time -= h;
		time = time/24;
		if (time > 0) {
			d = time;
		}
		return `${(d+m+h) !== 0 ? '-P' : ''}${d !== 0 ? d+'D' : ''}${h !== 0 || m !== 0 ? 'T' : ''}${h !== 0 ? h + 'H' : ''}${m !== 0 ? m + 'M' : ''}`;
	},

	converToJCALString: function (str) {
		var re = /^([\d]{8})(T)?([\d]{6})?(Z)?$/,
			date = '',
			time = '',
			result = re.exec(str);

		if (!result) {
			return str;
		}
		if (result[1]) {
			date = result[1].replace(/([\d]{4})([\d]{2})([\d]{2})/, function(all, year, month, day) {
				return `${year}-${month}-${day}`;
			});
		}
		if (result[3]) {
			time = 'T' + result[3].replace(/([\d]{2})([\d]{2})([\d]{2})/, function(all, hour, min, sec) {
			return `${hour}:${min}:${sec}`;
		}) + (result[4] || '');
		}
		return date + time;
	},

	getZoneByTZId: function (tzid) {
		// FIXME
		var tzService = ICAL.TimezoneService;
		return tzService.get(tzid);
	},

	fromAllString: function (str) {
		if (str instanceof ICAL.Time) {
			return str;
		}

		var splitResult = str.split(':'),
			len = splitResult.length,
			timezone,
			timeStr,
			time;

		timeStr = splitResult[len - 1];
		time = (timeStr.length <= 10) ? ICAL.Time.fromDateString(this.converToJCALString(timeStr)) : ICAL.Time.fromDateTimeString(this.converToJCALString(timeStr));

		if (len > 2) {
			splitResult.splice(splitResult.length - 1, 1);
			splitResult = [splitResult.join(':'), timeStr];
		}

		if (len > 1) {
			splitResult[0] = splitResult[0].replace('TZID=', '').replace(/\"/g, '');
			timezone = this.getZoneByTZId(splitResult[0]);
		}

		if (timezone) {
			time.zone = timezone;
			time.timezone = splitResult[0];
			time.tzid = splitResult[0];
		}
		return time;
	}
};

var recurParser = {
	fromString: function(str) {
		str = this.convertUntil(str);
		return ICAL.Recur.fromString(str);
	},
	convertUntil: function(str) {
		return str.replace(/UNTIL=([^(;|$)]*)/, function(all, until) {
			return 'UNTIL=' + timeParser.converToJCALString(until);
		}.bind(this));
	}
}

var Adapter = {
	getLocalIp: function (ifaces) {
		var ip;
		Object.keys(ifaces).forEach(function (ifname) {
			var alias = 0;

			ifaces[ifname].forEach(function (iface) {
				if ('IPv4' !== iface.family || iface.internal !== false) {
						return;
				}
				if (alias >= 1) {
					//if there have multiple ips we should choose the using ip
					ip = iface.address;
				} else {
					ip = iface.address;
				}
				++alias;
			});
		});
		return ip;
	},

	random4: function() {
		return Math.floor((1 + Math.random()) * 0x10000)
		.toString(16)
		.substring(1);
	},

	generateGUID: function () {
		return ICAL.Time.fromJSDate(new Date()).toICALString() + '-' + this.random4() + this.random4() + '@'
			+ this.getLocalIp(OS.networkInterfaces());
	},

	formAlarm: function (data) {
		return[`BEGIN:VALARM`,
		`ACTION:${data.alarm_action || ''}`,
		`DESCRIPTION:This is an event reminder`,
		`TRIGGER;VALUE=${data.time_format || ''}:${data.time_value || ''}`,
		`END:VALARM`].join('\n');
	},

	updateAlarm: function(data) {
		if (!data.evt_notify_setting) {
			return;
		}

		data.evt_notify_setting.forEach(function(alarmObj) {
			if (alarmObj.alarm_ical) {
				var icalComp = ICAL.Component.fromString(alarmObj.alarm_ical);
				if (!icalComp.hasProperty('x-apple-structured-location')) {
					alarmObj.alarm_ical = this.formAlarm(alarmObj);
				}
			} else {
				alarmObj.alarm_ical = this.formAlarm(alarmObj);
			}
		}, this);
	},

	formTimezone: function(data) {
		if (!data.tz_ical) {
			return;
		}
		return ICAL.Component.fromString(data.tz_ical.replace(/\\r\\n/g, '\n'));
	},

	formVTodo: function(data) {
		var	uid = this.generateGUID(),
			date = ICAL.Time.fromJSDate(new Date()).toICALString(),
			location = data.location || '',
			description = data.description || '';

		var formDTStart = function(data) {
			return data.dtstart ? `DTSTART${data.is_all_day ? ';VALUE=DATE:' + (data.dtstart || '') : ';' + (data.dtstart || '')}` : '';
		}
		var formDue = function(data) {
			return data.due ? `DUE${data.is_all_day ? ';VALUE=DATE:' + (data.due || '') : ';' + (data.due || '')}` : '';
		}

		data['ical_uid'] = uid;
		return [`BEGIN:VTODO`,
		`CREATED:${date || ''}`,
		`LAST-MOD:${date || ''}`,
		`UID:${data.uid || uid || ''}`,
		`DTSTAMP:${date || ''}`,
		`SUMMARY:${data.summary || ''}`,
		`${formDTStart(data)}`,
		`${formDue(data)}`,
		`SEQUENCE:0`,
		`PRIORITY:${data.priority ? data.priority : 0}`,
		`PERCENT-COMPLETE:${data.percent_complete ? data.percent_complete : 0}`,
		`STATUS:${data.percent_complete === 100 ? 'COMPLETED' : 'NEEDS-ACTION'}`,
		`DESCRIPTION:${description.replace(/\r/g, '\\r').replace(/\n/g, '\\n')}`,
		`LOCATION:${location}`,
		`END:VTODO`].join('\r\n');
	},

	formRRule: function(repeat_rule) {
		var rruleObj;
		try {
			/* Google rrule is correct but ICAL can not parse correctly
			 * Google: FREQ=WEEKLY;UNTIL=20171004T020000Z;BYDAY=WE
			 * ICAL: FREQ=WEEKLY;BYDAY=WE;UNTIL=2017-06-01T00:00:00;WKST=SU*/
			rruleObj = ICAL.Recur.fromString(repeat_rule);
		} catch (err) {
			return repeat_rule;
		}
		if (rruleObj.until) {
			return rruleObj.toString().replace(rruleObj.until.toString(), rruleObj.until.toICALString());
		}
		return rruleObj.toString();
	},

	formVEvent: function (data) {
		var	uid = this.generateGUID(),
			date = ICAL.Time.fromJSDate(new Date()).toICALString(),
			location = data.location || '',
			description = data.description || '';

		if (data.is_all_day === null || data.is_all_day === undefined) {
			throw "is_all_day should not be empty";
		}
		if (true === data.is_repeat_evt && !data.repeat_rule) {
			throw "is_repeat_evt is true but repeat_rule is empty";
		}

		data['ical_uid'] = uid;
		return [`BEGIN:VEVENT`,
		`CREATED:${date || ''}`,
		`LAST-MODIFIED:${date || ''}`,
		`DTSTAMP:${date || ''}`,
		`UID:${data.uid || uid || ''}`,
		`SEQUENCE:${data.sequence || 0}`,
		`SUMMARY:${data.summary || ''}`,
		`TRANSP:${data.transp || ''}`,
		`DTSTART${data.is_all_day ? ';VALUE=DATE:' + (data.dtstart || '') : ';' + (data.dtstart || '')}`,
		`DTEND${data.is_all_day ? ';VALUE=DATE:' + (data.dtend || '') : ';' + (data.dtend || '')}`,
		/* If the string contains '\n', the ICAL fromString would fail.
		 * So it needs to escape first.
		 * However getting value from ICAL is unescaped automatically.*/
		`DESCRIPTION:${description.replace(/\r/g, '\\r').replace(/\n/g, '\\n')}`,
		`LOCATION:${location}${data.is_repeat_evt ? '\r\nRRULE:' + this.formRRule(data.repeat_rule) : ''}`,
		`END:VEVENT`].join('\r\n');
	},

	formIcal: function (data) {
		var vdata;

		if (typeof data === 'string' || data instanceof String) {
			vdata = data;
		} else if (data.dtend) {
			vdata = this.formVEvent(data);
		} else {
			vdata = this.formVTodo(data);
		}

		return [`BEGIN:VCALENDAR`,
		`PRODID:-//Synology//EN`,
		`VERSION:2.0`,
		`${vdata}`,
		`END:VCALENDAR`].join('\r\n');

	},

	formAttendeeIcal: function (attendeeEmail, action, status, organizerEmail) {
		status = status || '1.2';
		action = action || 'NEEDS-ACTION';
		return `ATTENDEE;${status === '1.2' ? 'RSVP=TRUE;' : ''}PARTSTAT=${action};ROLE=REQ-PARTICIPANT;SENT-BY="mailto:${organizerEmail}";SCHEDULE-STATUS=${status}:mailto:${attendeeEmail}`;
	},

	formOrganizeIcal: function (organizer) {
		return `ORGANIZER;PARTSTAT=ACCEPTED;ROLE=CHAIR:mailto:${organizer}`;
	},

	removeInvalidEvent: function (oldVcal, data) {
		if (data.skip_remove_subevent === true) {
			return;
		}
		var vevents = oldVcal.getAllSubcomponents('vevent'),
			oldRule = vevents[0].getFirstPropertyValue('rrule') ? vevents[0].getFirstPropertyValue('rrule').toString() : null,
			newRule = (data.repeat_rule || oldRule),
			rruleChanged,
			isAllDayChanged = data.is_all_day !== (!vevents[0].getFirstProperty('dtstart').getParameter('tzid')),
			oldZone,
			oldStart,
			newZone,
			newStart,
			startDiff = false,
			needRemoveEvent;

		if (newRule) {
			newRule = recurParser.convertUntil(newRule);
		}

		rruleChanged = newRule !== oldRule;
		if (data.dtstart) {
			oldZone = vevents[0].getFirstProperty('dtstart').getParameter('tzid') || '';
			oldStart =  vevents[0].getFirstProperty('dtstart').toICALString();
			newZone = data.tz_id || '';
			newStart = data.dtstart;
			startDiff = oldZone !== newZone || oldStart.indexOf(newStart) < 0;
		}

		needRemoveEvent = (rruleChanged || isAllDayChanged || startDiff);

		if(needRemoveEvent) {
			vevents.forEach(function (vevent) {
				if (vevent.hasProperty('recurrence-id')) {
					oldVcal.removeSubcomponent(vevent);
				}
			});
			return;
		}

		if (data.new_exdate) {
			var exdateTime = timeParser.fromAllString(data.new_exdate);
			vevents.every(function(vevent) {
				var recurrenceId = vevent.getFirstProperty('recurrence-id'),
					recurrenceVal,
					recurrenceZone;
				if (recurrenceId) {
					recurrenceZone = recurrenceId.getParameter('tzid');
					recurrenceVal = recurrenceId.getFirstValue();
					if (recurrenceZone) {
						recurrenceVal.zone = ICAL.TimezoneService.get(recurrenceZone);
						recurrenceVal.timezone = recurrenceZone;
					}

					if (recurrenceVal.compare(exdateTime) === 0) {
						oldVcal.removeSubcomponent(vevent);
						return false;
					}
				}
				return true;
			});
		}
	},

	initIcal: function(data) {
		var vcalendar = ICAL.Component.fromString(this.formIcal(data));
		this.updateIcal(data, vcalendar);
	},

	updateIcal: function (data, vcalendar) {
		if (data.api === 'SYNO.Cal.Event') {
			this.updateIcalEvent(data, vcalendar);
		} else {
			this.updateIcalTodo(data, vcalendar);
		}
	},

	updateForReply: function(data) {
		var recurrenceId = data['response-recurrence-id'];
		if (!recurrenceId) {
			return data;
		}
		data = this.constructReplyVevt(data, recurrenceId);
		data = this.constructReplyArray(data, recurrenceId);
		return data;
	},

	shallowCopy: function(input) {
		var output = {};
		for (var i in input) {
			if (input.hasOwnProperty(i)) {
				output[i] = input[i];
			}
		}
		return output;
	},

	constructReplyArray: function(data, recurrenceId) {
		var attendeeArr = data.evt_attendee,
			matchedAttendeeArr = attendeeArr.filter(function(obj) {
				return obj['recurrence-id'] === recurrenceId;
			}),
			mainAttendeeArr = attendeeArr.filter(function(obj) {
				return !obj['recurrence-id'];
			});
		if (matchedAttendeeArr.length > 0) {
			return;
		}

		data.evt_attendee = data.evt_attendee.concat(mainAttendeeArr.map(function(obj) {
			var copyObj = this.shallowCopy(obj);
			copyObj['recurrence-id'] = recurrenceId;
			return copyObj;
		}.bind(this)));
		return data;
	},

	constructReplyVevt: function(data, recurrenceId) {
		var vcal,
			vevents,
			hasMatched = false,
			isAllDay = data['response-allday'],
			dtstart = data['response-dtstart'],
			dtend = data['response-dtend'],
			starttz = data['response-starttz'],
			endtz = data['response-endtz'];

		vcal = new ICAL.Component.fromString(data.evt_ical.replace(/\\r\\n/g, '\n'));
		vevents = vcal.getAllSubcomponents('vevent');
		vevents.every(function (evt) {
			var target = evt.getFirstPropertyValue('recurrence-id');
			if (target && target.toICALString() === recurrenceId) {
				hasMatched = true;
				return false;
			}
			return true;
		});

		if (!hasMatched) {
			if (vevents[0]) {
				var vevent = ICAL.Component.fromString(vevents[0].toString());
				vevent.addPropertyWithValue('recurrence-id', timeParser.fromAllString(recurrenceId));
				vevent.updatePropertyWithValue('dtstart', timeParser.fromAllString(dtstart));
				vevent.updatePropertyWithValue('dtend', timeParser.fromAllString(dtend));

				/*if (isAllDay) {
					vevent.getFirstProperty('dtstart').removeParameter('tzid');
					vevent.getFirstProperty('dtend').removeParameter('tzid');
				} else {
					vevent.getFirstProperty('dtstart').setParameter('tzid', starttz);
					vevent.getFirstProperty('dtend').setParameter('tzid', endtz);
				}*/

				var prop = vevent.getFirstProperty('recurrence-id');
				if (!data.is_all_day) {
					prop.setParameter('tzid', data.tz_id);
				}
				vevent.removeProperty('rrule');

				vcal.addSubcomponent(vevent);
				data.is_new_subevt = true;
				data.evt_ical = vcal.toString();
			}
		}
		return data;
	},

	updateSequece: function(vcomp) {
		var seqVal = parseInt(vcomp.getFirstPropertyValue('sequence'), 10) + 1 || 0;
		vcomp.updatePropertyWithValue('sequence', seqVal);
	},

	updateEventStatus: function(vcalendar, vcomp, method) {
		this.updateSequece(vcomp);

		if(method === 'CANCEL') {
			vcalendar.updatePropertyWithValue('method', 'CANCEL');
			vcomp.updatePropertyWithValue('status', 'CANCELLED');
		}
	},

	updateIcalTodo: function (data, vcalendar) {
		var	vtodos = vcalendar.getAllSubcomponents('vtodo'),
			zone = data.tz_id, //Is tz_id equal to the dtsart and dtend's time zone?
			vtodo;

		vtodo = vtodos[0];

		if (!vtodo) {
			throw 'no event match!';
		}

		vtodo.updatePropertyWithValue('percent_complete', data['percent_complete'] || 0);
		if(data['percent_complete'] && data['percent_complete'] == 100) {
			vtodo.updatePropertyWithValue('status', 'COMPLETED');
		} else {
			vtodo.updatePropertyWithValue('status', 'NEEDS-ACTION');
			vtodo.removeProperty('completed');
		}

		this.updateSequece(vtodo);
		this.processDefaultInfo(vtodo, data);
		this.processTimeInfo(vtodo, data);
		this.processTimezoneInfo(data, vcalendar);
		this.updateAlarm(data);
		this.removeUnUsedFields(data);

		data.evt_ical = vcalendar.toString();
	},

	getNeedProcessEvent: function(data, vcalendar) {
		var	vevents = vcalendar.getAllSubcomponents('vevent'),
			recurrenceId = data['recurrence-id'],
			zone = data.tz_id, //Is tz_id equal to the dtsart and dtend's time zone?
			vevent;

		if (recurrenceId) {
			if (vevents.length === 1 && !vevents[0].hasProperty('rrule')) {
				throw 'no repeat rule!';
			}
			vevents.every(function (evt) {
				var target = evt.getFirstPropertyValue('recurrence-id');
				if (target && target.toICALString() === recurrenceId) {
					vevent = evt;
					return false;
				}
				return true;
			});

			if (!vevent) {
				if (vevents[0]) {
					vevent = ICAL.Component.fromString(vevents[0].toString());
					vevent.updatePropertyWithValue('recurrence-id', timeParser.fromAllString(recurrenceId));

					// remove internal attachments only, preserve the external ones.
					(vevent.getAllProperties('attach') || []).forEach(function(attach) {
						if (!attach.getParameter('x-syno-rel-uri')) {
							return;
						}
						vevent.removeProperty(attach);
					});

					var prop = vevent.getFirstProperty('recurrence-id');
					if (!data.is_all_day) {
						prop.setParameter('tzid', zone);
					}
					vevent.removeProperty('rrule');
				} else {
					throw 'no match and no parent event!';
				}
				data.is_new_subevt = true;
			}

			vcalendar.addSubcomponent(vevent);
		} else {
			vevent = vevents[0];
			this.removeInvalidEvent(vcalendar, data);
		}

		if (!vevent) {
			throw 'no event match!';
		}

		return vevent;
	},

	updateIcalEvent:function (data, vcalendar) {
		var	vevents = vcalendar.getAllSubcomponents('vevent'),
			vevent;

		vevent = this.getNeedProcessEvent(data, vcalendar);
		this.updateSequece(vevent);
		this.processDefaultInfo(vevent, data);
		this.processAttachments(vevent, data);
		this.processTimeInfo(vevent, data);
		this.processAttendeeInfo(vevent, vevents, data);
		this.processTimezoneInfo(data, vcalendar);
		this.updateAlarm(data);
		this.removeUnUsedFields(data);

		data.evt_ical = vcalendar.toString();
	},

	updateUID: function(vevent, newUID) {
		if (vevent.hasProperty('uid')) {
			var uid = vevent.getFirstProperty('uid');
			uid.setValue(newUID);
		} else {
			vevent.addPropertyWithValue('uid', newUID);
		}
	},

	updateExdate: function(vevent, exdateTime) {
		if (vevent.hasProperty('exdate')) {
			var exdatesArr = vevent.getAllProperties('exdate'),
				exdateValueArr = [],
				i, j;

			for (i = 0; i < exdatesArr.length; i++) {
				var values = exdatesArr[i].getValues(),
					tmpTZID = exdatesArr[i].getParameter('tzid'),
					icalTimeStr;
				for (j = 0; j < values.length; j++) {
					if (values[j].timezone !== 'Z') {
						icalTimeStr = timeParser.fromAllString('TZID=' + tmpTZID + ':' + values[j].toICALString()).convertToZone(ICAL.Timezone.utcTimezone).toICALString();
					} else {
						icalTimeStr = values[j].toICALString();
					}

					// preserve unique time string only
					if (exdateValueArr.indexOf(icalTimeStr) < 0) {
						exdateValueArr.push(icalTimeStr);
					}
				}
			}

			// convert from String to ICAL.Time
			exdateValueArr = exdateValueArr.map(function(str, index) {
				return timeParser.fromAllString(str);
			});

			exdateValueArr.push(exdateTime);

			vevent.removeAllProperties('exdate');
			for (i = 0; i < exdateValueArr.length; i++) {
				vevent.addPropertyWithValue('exdate', exdateValueArr[i]);
			}
		} else {
			vevent.addPropertyWithValue('exdate', exdateTime);
		}
	},

	processDefaultInfo: function(vcomp, data) {
		['summary', 'description', 'location', 'transp'].forEach(function(key) {
			if (data.hasOwnProperty(key)) {
				vcomp.updatePropertyWithValue(key, data[key] || '');
			}
		});
		if (data.is_repeat_evt && !data['recurrence-id']) {
			vcomp.updatePropertyWithValue('rrule', recurParser.fromString(data.repeat_rule).toJSON());
			if (data.new_exdate) {
				// update exdate value
				this.updateExdate(vcomp, timeParser.fromAllString(data.new_exdate));
			}
		} else {
			vcomp.removeAllProperties('rrule');
			vcomp.removeAllProperties('exdate');
		}
		if (data.evt_color) {
			vcomp.updatePropertyWithValue('x-syno-evt-color', data.evt_color);
		}

		data.gps = data.gps || null;
		if (!data.gps) {
			vcomp.removeProperty('x-apple-structured-location');
		} else if (data.gps) {
			var location = vcomp.updatePropertyWithValue('x-apple-structured-location', `geo:${data.gps.gps.lat},${data.gps.gps.lng}`);
			location.setParameter('value', 'URI');
			location.setParameter('x-title', `${data.gps.name}`);
		}

		if (data.is_repeat_evt) {
			data.evt_repeat_setting = { repeat_rule: data.repeat_rule };
		}
	},

	processAttachments: function(vcomp, data) {
		var curAttachments = vcomp.getAllProperties('attach');

		if (data.attachments_remove && data.attachments_remove.length > 0) {
			data.attachments_remove.forEach(function(uriObj) {
				var file_uri = uriObj.file_uri;
				curAttachments.forEach(function(attach) {
					if (file_uri === attach.getFirstValue()) {
						vcomp.removeProperty(attach);
					};
				}, this);
			}, this);
		}
		if (data.attachments_add && data.attachments_add.length > 0) {
			data.attachments_add.forEach(function(uriObj) {
				var file_uri = uriObj.file_uri,
					file_rel_uri = uriObj.file_rel_uri,
					file_size = uriObj.file_size + '',
					p;

				if (!file_uri) {
					return;
				}
				p = ICAL.Property.fromString(`ATTACH:${file_uri}`);
				p.setParameter('type', 'URI');
				if (file_rel_uri) {
					p.setParameter('X-SYNO-REL-URI', file_rel_uri);
				}
				if (file_size) {
					p.setParameter('X-SYNO-FILE-SIZE', file_size);
				}
				vcomp.addProperty(p);
			});
		}
	},

	processTimeInfo: function(vcomp, data) {
		['dtstart', 'dtend', 'recurrence-id', 'due'].forEach(function(key) {
			if (!data[key]) {
				return;
			}

			var prop = vcomp.getFirstProperty(key),
				val = timeParser.fromAllString(data[key]);

			if (!prop) {
				vcomp.addPropertyWithValue(key, val);
				prop = vcomp.getFirstProperty(key);
			} else {
				vcomp.updatePropertyWithValue(key, val);
			}

			if (data.is_all_day) {
				prop.removeParameter('tzid');
			} else {
				prop.setParameter('tzid', data.tz_id);
			}
		}, this);
		if (!data.dtstart && !data.due) {
			vcomp.removeProperty('dtstart');
			vcomp.removeProperty('due');
		}
	},

	formEmailByName: function(name) {
		var domainPostfix = '.domain',
			domainReg = /([^\\]+)\\([^\\]+)/,
			topDomainReg = /@[^\.]+\./,
			domainArr = domainReg.exec(name),
			localPostfix = '@local.host',
			email;

		if (!!domainArr) {
			email = domainArr[2] + '@' + domainArr[1] + domainPostfix;
		} else {
			email = name.indexOf('@') < 0 ? name + localPostfix : name;

			if (!topDomainReg.test(email)) {
				email += '.host';
			}
		}
		return email;
	},

	processAttendeeInfo: function(vevent, vevents, data) {
		data.user_add = data.user_add || [];
		data.user_remove = data.user_remove || [];
		data.email_add = data.email_add || [];
		data.email_remove = data.email_remove || [];

		if (!data.evt_attendee ||
			(data.user_add.length === 0 && data.user_remove.length === 0 && data.email_add.length === 0 && data.email_remove.length === 0)) {
			return;
		}

		var target = (data['recurrence-id']) ? [vevent] : vevents,
			organizer = data.evt_attendee.filter(function(attendee) {
				if (attendee.invite_actor === 'ORGANIZER') {
					return true;
				}
				return false;
			})[0],
			filterFn = function(filterArrKey, filterAttrKey, attendee) {
				if (data[filterArrKey] && data[filterArrKey].indexOf(attendee[filterAttrKey]) >= 0) {
					return true;
				}
				return false;
			},
			attendeeKeyMap = {},
			addUserFn = function(v, organizerEmail, attendee) {
				var oEmail = organizerEmail || '';
				if (attendeeKeyMap[attendee.email]) {
					return;
				}
				v.addProperty(ICAL.Property.fromString(this.formAttendeeIcal(attendee.email, attendee.invite_status, attendee.invite_status === 'NEEDS-ACTION' ? '1.2' : '2.0', oEmail)));
			},
			delUserFn = function(v, attendee) {
				var delAttendee = v.getAllProperties('attendee').filter(function(attendeeProp) {
					if (attendeeProp.getFirstValue() === 'mailto:' + attendee) {
						return true;
					}
					return false;
				});
				delAttendee.forEach(function(attendeeProp) {
					v.removeProperty(attendeeProp);
				});
			},
			addUserAttendee = data.evt_attendee.filter(filterFn.bind(this, 'user_add', 'ug_name')),
			addEmailAttendee = data.evt_attendee.filter(filterFn.bind(this, 'email_add', 'email')),
			delUserAttendee = data.user_remove.map(this.formEmailByName),
			delEmailAttendee = data.email_remove.map(this.formEmailByName),
			oEmail = (organizer && organizer.email) || '';

		target.forEach(function(v) {
			// check organizer
			if (!v.getFirstProperty('organizer')) {
				v.addProperty(ICAL.Property.fromString(this.formOrganizeIcal(oEmail)));
			}
			attendeeKeyMap = {};
			var curAttendees = v.getAllProperties('attendee');
			curAttendees.map(function(attendee, idx) {
				attendeeKeyMap[attendee.getFirstValue().replace('mailto:', '')] = true;
			});
			// user add
			addUserAttendee.map(addUserFn.bind(this, v, oEmail));
			// email add
			addEmailAttendee.map(addUserFn.bind(this, v, oEmail));
			// user remove
			delUserAttendee.map(delUserFn.bind(this, v));
			// email remove
			delEmailAttendee.map(delUserFn.bind(this, v));

			if (curAttendees.length > 0 && (curAttendees.length + addUserAttendee.length + addEmailAttendee.length) === (delUserAttendee.length + delEmailAttendee.length)) {
				v.removeProperty(v.getFirstProperty('organizer'));
			}
		}, this);
	},

	processTimezoneInfo: function(data, vcalendar) {
		if (data.is_all_day || (!data.due && !data.dtstart)) {
			// no timezone info should be included in vcalendar
			vcalendar.removeSubcomponent('vtimezone');
		} else {
			var vtimezone = vcalendar.getFirstSubcomponent('vtimezone'),
				tzid;
			if (vtimezone) {
				tzid = vtimezone.getFirstPropertyValue('tzid');
				// tzid changed, remove old one, add new one
				if (tzid != data.tz_id) {
					vcalendar.removeSubcomponent('vtimezone');
					vcalendar.addSubcomponent(this.formTimezone(data));
				}
			} else {
				// no timezone info before, add timezone info straight into vcalendar
				vcalendar.addSubcomponent(this.formTimezone(data));
			}
		}
	},

	removeUnUsedFields: function(data) {
		delete data.tz_ical;
		delete data.repeat_rule;
	},

	registerTimezone: function(tzStr, tzid) {
		var tzService = ICAL.TimezoneService;
		if (!tzService.has(tzid)) {
			tzService.register(tzid, new ICAL.Timezone({
				component: tzStr,
				tzid: tzid
			}));
		}
	},

	registerExistTimezone: function(vcal) {
		var vtimezone = vcal.getFirstSubcomponent('vtimezone'),
			tzid = vtimezone ? vtimezone.getFirstPropertyValue('tzid') : null;

		if (!tzid) {
			return;
		}
		this.registerTimezone(vtimezone.toString(), tzid);
	},

	update: function(data, isNew) {
		if (data.tzid) {
			this.registerTimezone(data.tz_ical, data.tzid);
		}

		if (isNew) {
			this.initIcal(data);
		} else {
			var vcal = new ICAL.Component.fromString(data.evt_ical.replace(/\\r\\n/g, '\n'));
			this.registerExistTimezone(vcal);
			this.updateIcal(data, vcal);
		}
		data.class = 'PUBLIC';

		return data;
	},

	getCancelIcs: function(data) {
		var vcal = new ICAL.Component.fromString(data.ics.replace(/\\r\\n/g, '\n'));
		var vevent = vcal.getFirstSubcomponent('vevent');

		this.updateEventStatus(vcal, vevent, 'CANCEL');
		data.ics = vcal.toString();

		return data;
	},


	updateAttachments: function(data) {
		var vcal = new ICAL.Component.fromString(data.evt_ical.replace(/\\r\\n/g, '\n')),
			vevent = this.getNeedProcessEvent(data, vcal);

		this.processAttachments(vevent, data);
		data.evt_ical = vcal.toString();
		return data;
	},

	registerDefaultTimezone:function () {
		var tzService = ICAL.TimezoneService;
		var content = fs.readFileSync(path.join(__dirname, 'timezone.json'));
		var timezones = JSON.parse(content);
		timezones.data.forEach(function(item) {
			var tzid = item.tzid,
				component = item.vtimezone;
			this.registerTimezone(component, tzid);
		}, this);
	}
};

module.exports = {
	update: Adapter.update.bind(Adapter),
	getCancelIcs : Adapter.getCancelIcs.bind(Adapter),
	initIcal: Adapter.initIcal.bind(Adapter),
	formVEvent: Adapter.formVEvent.bind(Adapter),
	formIcal: Adapter.formIcal.bind(Adapter),
	formAlarm: Adapter.formAlarm.bind(Adapter),
	updateExdate: Adapter.updateExdate.bind(Adapter),
	convertToHourDay: timeParser.convertToHourDay.bind(timeParser),
	fromAllString: timeParser.fromAllString.bind(timeParser),
	registerDefaultTimezone: Adapter.registerDefaultTimezone.bind(Adapter),
	generateGUID: Adapter.generateGUID.bind(Adapter),
	updateUID: Adapter.updateUID.bind(Adapter),
	formOrganizeIcal: Adapter.formOrganizeIcal.bind(Adapter),
	formAttendeeIcal: Adapter.formAttendeeIcal.bind(Adapter),
	getZoneByTZId: timeParser.getZoneByTZId.bind(timeParser),
	random4: Adapter.random4.bind(Adapter),
	updateForReply: Adapter.updateForReply.bind(Adapter),
	updateAttachments: Adapter.updateAttachments.bind(Adapter)
};
